@@ -36,7 +36,7 @@ use base qw(
     Clownfish::CFC::Perl::Build::Charmonic
 );
 
-our $VERSION = '0.004004';
+our $VERSION = '0.004099_001';
 $VERSION = eval $VERSION;
 
 use File::Spec::Functions qw( catdir catfile updir rel2abs );
@@ -65,7 +65,7 @@ else {
 
 sub new {
     my ( $class, %args ) = @_;
-    $args{include_dirs}     = $XS_SOURCE_DIR;
+    $args{include_dirs}     = [ $CORE_SOURCE_DIR, $XS_SOURCE_DIR ];
     $args{clownfish_params} = {
         autogen_header => _autogen_header(),
         include        => [],                  # Don't use default includes.
@@ -161,6 +161,14 @@ sub ACTION_compile_custom_xs {
         $self->clownfish_params( cflags => $cf_cflags );
     }
 
+    # Add extra linker flags from Charmonizer.
+    my $charm_ldflags = $self->charmony('EXTRA_LDFLAGS');
+    if ($charm_ldflags) {
+        my $extra_ldflags = $self->extra_linker_flags;
+        push @$extra_ldflags, $self->split_like_shell($charm_ldflags);
+        $self->extra_linker_flags(@$extra_ldflags);
+    }
+
     $self->SUPER::ACTION_compile_custom_xs;
 }
 
@@ -246,30 +254,28 @@ sub ACTION_test_valgrind {
 
 sub _autogen_header {
     return <<"END_AUTOGEN";
-/***********************************************
+***********************************************
 
- !!!! DO NOT EDIT !!!!
+!!!! DO NOT EDIT !!!!
 
- This file was auto-generated by Build.PL.
+This file was auto-generated by Build.PL.
 
- ***********************************************/
+***********************************************
 
-/* Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
- * this work for additional information regarding copyright ownership.
- * The ASF licenses this file to You under the Apache License, Version 2.0
- * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
+Licensed to the Apache Software Foundation (ASF) under one or more
+contributor license agreements.  See the NOTICE file distributed with
+this work for additional information regarding copyright ownership.
+The ASF licenses this file to You under the Apache License, Version 2.0
+(the "License"); you may not use this file except in compliance with
+the License.  You may obtain a copy of the License at
 
+    http://www.apache.org/licenses/LICENSE-2.0
+
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 END_AUTOGEN
 }
 
@@ -298,12 +304,12 @@ sub ACTION_dist {
     # the top-level $REPOS_ROOT.  Because some assets we need are outside this
     # directory, we need to copy them in.
     my %to_copy = (
-        '../../CONTRIBUTING' => 'CONTRIBUTING',
-        '../../LICENSE'      => 'LICENSE',
-        '../../NOTICE'       => 'NOTICE',
-        '../../README'       => 'README',
-        $CORE_SOURCE_DIR     => 'core',
-        $CHARMONIZER_C       => 'charmonizer.c',
+        '../../CONTRIBUTING.md' => 'CONTRIBUTING.md',
+        '../../LICENSE'         => 'LICENSE',
+        '../../NOTICE'          => 'NOTICE',
+        '../../README.md'       => 'README.md',
+        $CORE_SOURCE_DIR        => 'core',
+        $CHARMONIZER_C          => 'charmonizer.c',
     );
     print "Copying files...\n";
     while ( my ( $from, $to ) = each %to_copy ) {
