@@ -16,22 +16,27 @@ package Clownfish::Build::Binding;
 use strict;
 use warnings;
 
-our $VERSION = '0.004004';
+our $VERSION = '0.004099_001';
 $VERSION = eval $VERSION;
 
 sub bind_all {
     my $class = shift;
     $class->bind_clownfish;
     $class->bind_test;
-    $class->bind_test_alias_obj;
+    $class->bind_test_host;
+    $class->bind_blob;
+    $class->bind_boolean;
     $class->bind_bytebuf;
+    $class->bind_charbuf;
     $class->bind_string;
+    $class->bind_stringiterator;
     $class->bind_err;
     $class->bind_hash;
-    $class->bind_float32;
-    $class->bind_float64;
+    $class->bind_hashiterator;
+    $class->bind_float;
+    $class->bind_integer;
     $class->bind_obj;
-    $class->bind_varray;
+    $class->bind_vector;
     $class->bind_class;
     $class->bind_stringhelper;
 }
@@ -40,38 +45,15 @@ sub bind_clownfish {
     my $xs_code = <<'END_XS_CODE';
 MODULE = Clownfish    PACKAGE = Clownfish
 
-IV
-_dummy_function()
-CODE:
-    RETVAL = 1;
-OUTPUT:
-    RETVAL
-
 SV*
 to_clownfish(sv)
     SV *sv;
 CODE:
 {
-    cfish_Obj *obj = XSBind_perl_to_cfish(sv);
+    cfish_Obj *obj = XSBind_perl_to_cfish_nullable(aTHX_ sv, CFISH_OBJ);
     RETVAL = CFISH_OBJ_TO_SV_NOINC(obj);
 }
 OUTPUT: RETVAL
-
-SV*
-to_perl(sv)
-    SV *sv;
-CODE:
-{
-    if (sv_isobject(sv) && sv_derived_from(sv, "Clownfish::Obj")) {
-        IV tmp = SvIV(SvRV(sv));
-        cfish_Obj* obj = INT2PTR(cfish_Obj*, tmp);
-        RETVAL = XSBind_cfish_to_perl(obj);
-    }
-    else {
-        RETVAL = newSVsv(sv);
-    }
-}
-OUTPUT: RETVAL
 END_XS_CODE
 
     my $binding = Clownfish::CFC::Binding::Perl::Class->new(
@@ -88,19 +70,29 @@ sub bind_test {
 MODULE = Clownfish   PACKAGE = Clownfish::Test
 
 bool
-run_tests(package)
-    char *package;
+run_tests(class_name)
+    cfish_String *class_name;
 CODE:
-    cfish_String *class_name = cfish_Str_newf("%s", package);
     cfish_TestFormatter *formatter
         = (cfish_TestFormatter*)cfish_TestFormatterTAP_new();
     cfish_TestSuite *suite = testcfish_Test_create_test_suite();
-    bool result = CFISH_TestSuite_Run_Batch(suite, class_name, formatter);
-    CFISH_DECREF(class_name);
+    RETVAL = CFISH_TestSuite_Run_Batch(suite, class_name, formatter);
     CFISH_DECREF(formatter);
     CFISH_DECREF(suite);
+OUTPUT: RETVAL
 
-    RETVAL = result;
+void
+invoke_to_string(obj)
+    cfish_Obj *obj;
+PPCODE:
+    cfish_String *str = CFISH_Obj_To_String(obj);
+    CFISH_DECREF(str);
+
+int
+refcount(obj)
+    cfish_Obj *obj;
+CODE:
+    RETVAL = (int)CFISH_REFCOUNT_NN(obj);
 OUTPUT: RETVAL
 END_XS_CODE
 
@@ -113,10 +105,10 @@ END_XS_CODE
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
-sub bind_test_alias_obj {
+sub bind_test_host {
     my $binding = Clownfish::CFC::Binding::Perl::Class->new(
         parcel     => "TestClownfish",
-        class_name => "Clownfish::Test::AliasTestObj",
+        class_name => "Clownfish::Test::TestHost",
     );
     $binding->bind_method(
         alias  => 'perl_alias',
@@ -125,7 +117,120 @@ sub bind_test_alias_obj {
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
+sub bind_blob {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $blob = Clownfish::Blob->new($byte_string);
+    my $byte_string = $blob->to_perl;
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+=head2 new
+
+    my $blob = Clownfish::Blob->new($byte_string);
+
+Create a Blob containing the passed-in bytes.
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( pod => $constructor );
+
+    my $xs_code = <<'END_XS_CODE';
+MODULE = Clownfish     PACKAGE = Clownfish::Blob
+
+SV*
+new(either_sv, sv)
+    SV *either_sv;
+    SV *sv;
+CODE:
+{
+    STRLEN size;
+    char *ptr = SvPV(sv, size);
+    cfish_Blob *self
+        = (cfish_Blob*)XSBind_new_blank_obj(aTHX_ either_sv);
+    cfish_Blob_init(self, ptr, size);
+    RETVAL = CFISH_OBJ_TO_SV_NOINC(self);
+}
+OUTPUT: RETVAL
+END_XS_CODE
+
+    my $binding = Clownfish::CFC::Binding::Perl::Class->new(
+        parcel     => "Clownfish",
+        class_name => "Clownfish::Blob",
+    );
+    $binding->set_pod_spec($pod_spec);
+    $binding->append_xs($xs_code);
+    $binding->exclude_constructor;
+
+    Clownfish::CFC::Binding::Perl::Class->register($binding);
+}
+
+sub bind_boolean {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    use Clownfish::Boolean qw( $true_singleton $false_singleton );
+
+    my $bool = Clownfish::Boolean->singleton($truth_value);
+    my $truth_value = $bool->get_value;
+
+    if ($bool->equals($true_singleton)) {
+        print "true\n";
+    }
+END_SYNOPSIS
+    my $description = <<'END_DESCRIPTION';
+There are only two singleton instances of this class: C<$true_singleton> and
+C<$false_singleton> which are exported on demand.
+END_DESCRIPTION
+    my $constructor = <<'END_CONSTRUCTOR';
+=head2 singleton
+
+    my $bool = Clownfish::Boolean->singleton($truth_value);
+
+Return either C<$true_singleton> or C<$false_singleton> depending on the
+supplied value.
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->set_description($description);
+    $pod_spec->add_constructor( pod => $constructor );
+
+    my $xs_code = <<'END_XS_CODE';
+MODULE = Clownfish   PACKAGE = Clownfish::Boolean
+
+SV*
+singleton(either_sv, value)
+    SV      *either_sv;
+    bool     value;
+CODE:
+{
+    RETVAL = CFISH_OBJ_TO_SV_INC(cfish_Bool_singleton(value));
+}
+OUTPUT: RETVAL
+END_XS_CODE
+
+    my $binding = Clownfish::CFC::Binding::Perl::Class->new(
+        parcel     => "Clownfish",
+        class_name => "Clownfish::Boolean",
+    );
+    $binding->set_pod_spec($pod_spec);
+    $binding->append_xs($xs_code);
+
+    Clownfish::CFC::Binding::Perl::Class->register($binding);
+}
+
 sub bind_bytebuf {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $buf = Clownfish::ByteBuf->new($byte_string);
+    my $byte_string = $buf->to_perl;
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+=head2 new
+
+    my $buf = Clownfish::ByteBuf->new($byte_string);
+
+Create a ByteBuf containing the passed-in bytes.
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( pod => $constructor );
+
     my $xs_code = <<'END_XS_CODE';
 MODULE = Clownfish     PACKAGE = Clownfish::ByteBuf
 
@@ -137,9 +242,9 @@ CODE:
 {
     STRLEN size;
     char *ptr = SvPV(sv, size);
-    cfish_ByteBuf *self = (cfish_ByteBuf*)XSBind_new_blank_obj(either_sv);
-    cfish_BB_init(self, size);
-    CFISH_BB_Mimic_Bytes(self, ptr, size);
+    cfish_ByteBuf *self
+        = (cfish_ByteBuf*)XSBind_new_blank_obj(aTHX_ either_sv);
+    cfish_BB_init_bytes(self, ptr, size);
     RETVAL = CFISH_OBJ_TO_SV_NOINC(self);
 }
 OUTPUT: RETVAL
@@ -149,13 +254,52 @@ END_XS_CODE
         parcel     => "Clownfish",
         class_name => "Clownfish::ByteBuf",
     );
+    $binding->set_pod_spec($pod_spec);
     $binding->append_xs($xs_code);
     $binding->exclude_constructor;
 
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
+sub bind_charbuf {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $buf = Clownfish::CharBuf->new;
+    $buf->cat('abc');
+    $buf->cat_char(ord("\n"));
+    print $buf->to_string;
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+    my $buf = Clownfish::CharBuf->new( capacity => 256 );
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( sample => $constructor );
+
+    my $binding = Clownfish::CFC::Binding::Perl::Class->new(
+        parcel     => "Clownfish",
+        class_name => "Clownfish::CharBuf",
+    );
+    $binding->set_pod_spec($pod_spec);
+
+    Clownfish::CFC::Binding::Perl::Class->register($binding);
+}
+
 sub bind_string {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $string = Clownfish::String->new('abc');
+    print $string->to_perl, "\n";
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+=head2 new
+
+    my $string = Clownfish::String->new($perl_string);
+
+Return a String containing the passed-in Perl string.
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( pod => $constructor );
+
     my $xs_code = <<'END_XS_CODE';
 MODULE = Clownfish     PACKAGE = Clownfish::String
 
@@ -167,37 +311,122 @@ CODE:
 {
     STRLEN size;
     char *ptr = SvPVutf8(sv, size);
-    cfish_String *self = (cfish_String*)XSBind_new_blank_obj(either_sv);
+    cfish_String *self = (cfish_String*)XSBind_new_blank_obj(aTHX_ either_sv);
     cfish_Str_init_from_trusted_utf8(self, ptr, size);
     RETVAL = CFISH_OBJ_TO_SV_NOINC(self);
 }
 OUTPUT: RETVAL
-
-SV*
-_clone(self)
-    cfish_String *self;
-CODE:
-    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_Str_Clone_IMP(self));
-OUTPUT: RETVAL
-
-SV*
-to_perl(self)
-    cfish_String *self;
-CODE:
-    RETVAL = XSBind_str_to_sv(self);
-OUTPUT: RETVAL
 END_XS_CODE
 
     my $binding = Clownfish::CFC::Binding::Perl::Class->new(
         parcel     => "Clownfish",
         class_name => "Clownfish::String",
     );
+    $binding->set_pod_spec($pod_spec);
     $binding->append_xs($xs_code);
     $binding->exclude_constructor;
 
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
+sub bind_stringiterator {
+    my @hand_rolled = qw(
+        Next
+        Prev
+    );
+
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $iter = $string->top;
+    while (my $code_point = $iter->next) {
+        ...
+    }
+END_SYNOPSIS
+    my $next_pod = <<'END_POD';
+=head2 next
+
+    my $code_point = $iter->next;
+
+Return the code point after the current position and advance the
+iterator. Returns undef at the end of the string. Returns zero
+but true for U+0000.
+END_POD
+    my $prev_pod = <<'END_POD';
+=head2 prev
+
+    my $code_point = $iter->prev;
+
+Return the code point before the current position and go one step back.
+Returns undef at the start of the string. Returns zero but true for
+U+0000.
+END_POD
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_method(
+        method => 'Next',
+        alias  => 'next',
+        pod    => $next_pod,
+    );
+    $pod_spec->add_method(
+        method => 'Prev',
+        alias  => 'prev',
+        pod    => $prev_pod,
+    );
+
+    my $xs_code = <<'END_XS_CODE';
+MODULE = Clownfish   PACKAGE = Clownfish::StringIterator
+
+SV*
+next(self)
+    cfish_StringIterator *self;
+CODE:
+{
+    int32_t cp = CFISH_StrIter_Next(self);
+
+    if (cp == CFISH_STR_OOB) {
+        RETVAL = &PL_sv_undef;
+    }
+    else if (cp == 0) {
+        /* Zero but true. */
+        RETVAL = newSVpvn("0e0", 3);
+    }
+    else {
+        RETVAL = newSViv(cp);
+    }
+}
+OUTPUT: RETVAL
+
+SV*
+prev(self)
+    cfish_StringIterator *self;
+CODE:
+{
+    int32_t cp = CFISH_StrIter_Prev(self);
+
+    if (cp == CFISH_STR_OOB) {
+        RETVAL = &PL_sv_undef;
+    }
+    else if (cp == 0) {
+        /* Zero but true. */
+        RETVAL = newSVpvn("0e0", 3);
+    }
+    else {
+        RETVAL = newSViv(cp);
+    }
+}
+OUTPUT: RETVAL
+END_XS_CODE
+
+    my $binding = Clownfish::CFC::Binding::Perl::Class->new(
+        parcel     => "Clownfish",
+        class_name => "Clownfish::StringIterator",
+    );
+    $binding->set_pod_spec($pod_spec);
+    $binding->exclude_method($_) for @hand_rolled;
+    $binding->append_xs($xs_code);
+
+    Clownfish::CFC::Binding::Perl::Class->register($binding);
+}
+
 sub bind_err {
     my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
     my $synopsis = <<'END_SYNOPSIS';
@@ -250,49 +479,52 @@ END_XS_CODE
 sub bind_hash {
     my @hand_rolled = qw(
         Store
-        Next
+    );
+
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $hash = Clownfish::Hash->new;
+    $hash->store($key, $value);
+    my $value = $hash->fetch($key);
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+    my $hash = Clownfish::Hash->new( capacity => 256 );
+END_CONSTRUCTOR
+    my $store_pod = <<'END_POD';
+=head2 store
+
+    $hash->store($key, $value);
+
+Store a key-value pair.
+END_POD
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( sample => $constructor );
+    $pod_spec->add_method(
+        method => 'Store',
+        alias  => 'store',
+        pod    => $store_pod,
     );
 
     my $xs_code = <<'END_XS_CODE';
 MODULE = Clownfish    PACKAGE = Clownfish::Hash
 SV*
-_fetch(self, key)
+fetch_raw(self, key)
     cfish_Hash *self;
     cfish_String *key;
 CODE:
-    RETVAL = CFISH_OBJ_TO_SV(CFISH_Hash_Fetch_IMP(self, (cfish_Obj*)key));
+    RETVAL = CFISH_OBJ_TO_SV_INC(CFISH_Hash_Fetch_IMP(self, key));
 OUTPUT: RETVAL
 
 void
-store(self, key, value);
+store(self, key, value_sv);
     cfish_Hash         *self;
     cfish_String *key;
-    cfish_Obj          *value;
+    SV           *value_sv;
 PPCODE:
 {
-    if (value) { CFISH_INCREF(value); }
-    CFISH_Hash_Store_IMP(self, (cfish_Obj*)key, value);
-}
-
-void
-next(self)
-    cfish_Hash *self;
-PPCODE:
-{
-    cfish_Obj *key;
-    cfish_Obj *val;
-
-    if (CFISH_Hash_Next(self, &key, &val)) {
-        SV *key_sv = (SV*)CFISH_Obj_To_Host(key);
-        SV *val_sv = (SV*)CFISH_Obj_To_Host(val);
-
-        XPUSHs(sv_2mortal(key_sv));
-        XPUSHs(sv_2mortal(val_sv));
-        XSRETURN(2);
-    }
-    else {
-        XSRETURN_EMPTY;
-    }
+    cfish_Obj *value
+        = (cfish_Obj*)XSBind_perl_to_cfish_nullable(aTHX_ value_sv, CFISH_OBJ);
+    CFISH_Hash_Store_IMP(self, key, value);
 }
 END_XS_CODE
 
@@ -300,24 +532,40 @@ END_XS_CODE
         parcel     => "Clownfish",
         class_name => "Clownfish::Hash",
     );
+    $binding->set_pod_spec($pod_spec);
     $binding->exclude_method($_) for @hand_rolled;
     $binding->append_xs($xs_code);
 
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
-sub bind_float32 {
-    my $float32_xs_code = <<'END_XS_CODE';
-MODULE = Clownfish   PACKAGE = Clownfish::Float32
+sub bind_hashiterator {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $iter = Clownfish::HashIterator->new($hash);
+    while ($iter->next) {
+        my $key   = $iter->get_key;
+        my $value = $iter->get_value;
+    }
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+    my $iter = Clownfish::HashIterator->new($hash);
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( sample => $constructor );
+
+    my $xs_code = <<'END_XS_CODE';
+MODULE = Clownfish   PACKAGE = Clownfish::HashIterator
 
 SV*
-new(either_sv, value)
-    SV    *either_sv;
-    float  value;
+new(either_sv, hash)
+    SV         *either_sv;
+    cfish_Hash *hash;
 CODE:
 {
-    cfish_Float32 *self = (cfish_Float32*)XSBind_new_blank_obj(either_sv);
-    cfish_Float32_init(self, value);
+    cfish_HashIterator *self
+        = (cfish_HashIterator*)XSBind_new_blank_obj(aTHX_ either_sv);
+    cfish_HashIter_init(self, hash);
     RETVAL = CFISH_OBJ_TO_SV_NOINC(self);
 }
 OUTPUT: RETVAL
@@ -325,17 +573,29 @@ END_XS_CODE
 
     my $binding = Clownfish::CFC::Binding::Perl::Class->new(
         parcel     => "Clownfish",
-        class_name => "Clownfish::Float32",
+        class_name => "Clownfish::HashIterator",
     );
-    $binding->append_xs($float32_xs_code);
+    $binding->set_pod_spec($pod_spec);
+    $binding->append_xs($xs_code);
     $binding->exclude_constructor;
 
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
-sub bind_float64 {
-    my $float64_xs_code = <<'END_XS_CODE';
-MODULE = Clownfish   PACKAGE = Clownfish::Float64
+sub bind_float {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $float = Clownfish::Float->new(2.5);
+    my $value = $float->get_value;
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+    my $float = Clownfish::Float->new($value);
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( sample => $constructor );
+
+    my $xs_code = <<'END_XS_CODE';
+MODULE = Clownfish   PACKAGE = Clownfish::Float
 
 SV*
 new(either_sv, value)
@@ -343,8 +603,9 @@ new(either_sv, value)
     double  value;
 CODE:
 {
-    cfish_Float64 *self = (cfish_Float64*)XSBind_new_blank_obj(either_sv);
-    cfish_Float64_init(self, value);
+    cfish_Float *self
+        = (cfish_Float*)XSBind_new_blank_obj(aTHX_ either_sv);
+    cfish_Float_init(self, value);
     RETVAL = CFISH_OBJ_TO_SV_NOINC(self);
 }
 OUTPUT: RETVAL
@@ -352,25 +613,56 @@ END_XS_CODE
 
     my $binding = Clownfish::CFC::Binding::Perl::Class->new(
         parcel     => "Clownfish",
-        class_name => "Clownfish::Float64",
+        class_name => "Clownfish::Float",
     );
-    $binding->append_xs($float64_xs_code);
+    $binding->set_pod_spec($pod_spec);
+    $binding->append_xs($xs_code);
+    $binding->exclude_constructor;
+
+    Clownfish::CFC::Binding::Perl::Class->register($binding);
+}
+
+sub bind_integer {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $integer = Clownfish::Integer->new(7);
+    my $value = $integer->get_value;
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+    my $integer = Clownfish::Integer->new($value);
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( sample => $constructor );
+
+    my $xs_code = <<'END_XS_CODE';
+MODULE = Clownfish   PACKAGE = Clownfish::Integer
+
+SV*
+new(either_sv, value)
+    SV      *either_sv;
+    int64_t  value;
+CODE:
+{
+    cfish_Integer *self
+        = (cfish_Integer*)XSBind_new_blank_obj(aTHX_ either_sv);
+    cfish_Int_init(self, value);
+    RETVAL = CFISH_OBJ_TO_SV_NOINC(self);
+}
+OUTPUT: RETVAL
+END_XS_CODE
+
+    my $binding = Clownfish::CFC::Binding::Perl::Class->new(
+        parcel     => "Clownfish",
+        class_name => "Clownfish::Integer",
+    );
+    $binding->set_pod_spec($pod_spec);
+    $binding->append_xs($xs_code);
     $binding->exclude_constructor;
 
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
 sub bind_obj {
-    my @exposed = qw(
-        To_String
-        To_I64
-        To_F64
-        Equals
-    );
-    my @hand_rolled = qw(
-        Is_A
-    );
-
     my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
     my $synopsis = <<'END_SYNOPSIS';
     package MyObj;
@@ -439,28 +731,61 @@ results in a segfault rather than an exception.)
 
 =head1 CONSTRUCTOR
 
-=head2 new()
+=head2 new
+
+    my $self = $class->SUPER::new;
 
 Abstract constructor -- must be invoked via a subclass.  Attempting to
 instantiate objects of class "Clownfish::Obj" directly causes an
 error.
 
 Takes no arguments; if any are supplied, an error will be reported.
+END_DESCRIPTION
+    my $to_perl_pod = <<'END_POD';
+=head2 to_perl
 
-=head1 DESTRUCTOR
+    my $native = $obj->to_perl;
 
+Tries to convert the object to its native Perl representation.
+END_POD
+    my $destroy_pod = <<'END_POD';
 =head2 DESTROY
 
 All Clownfish classes implement a DESTROY method; if you override it in a
 subclass, you must call C<< $self->SUPER::DESTROY >> to avoid leaking memory.
-END_DESCRIPTION
+END_POD
     $pod_spec->set_synopsis($synopsis);
     $pod_spec->set_description($description);
-    $pod_spec->add_method( method => $_, alias => lc($_) ) for @exposed;
+    $pod_spec->add_method(
+        method => 'To_Host',
+        alias  => 'to_perl',
+        pod    => $to_perl_pod,
+    );
+    $pod_spec->add_method(
+        method => 'Destroy',
+        alias  => 'DESTROY',
+        pod    => $destroy_pod,
+    );
 
     my $xs_code = <<'END_XS_CODE';
 MODULE = Clownfish     PACKAGE = Clownfish::Obj
 
+SV*
+get_class(self)
+    cfish_Obj *self
+CODE:
+    cfish_Class *klass = cfish_Obj_get_class(self);
+    RETVAL = (SV*)CFISH_Class_To_Host(klass);
+OUTPUT: RETVAL
+
+SV*
+get_class_name(self)
+    cfish_Obj *self
+CODE:
+    cfish_String *class_name = cfish_Obj_get_class_name(self);
+    RETVAL = (SV*)CFISH_Str_To_Host(class_name);
+OUTPUT: RETVAL
+
 bool
 is_a(self, class_name)
     cfish_Obj *self;
@@ -468,9 +793,23 @@ is_a(self, class_name)
 CODE:
 {
     cfish_Class *target = cfish_Class_fetch_class(class_name);
-    RETVAL = CFISH_Obj_Is_A(self, target);
+    RETVAL = cfish_Obj_is_a(self, target);
 }
 OUTPUT: RETVAL
+
+SV*
+clone_raw(self)
+    cfish_Obj *self;
+CODE:
+    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_Obj_Clone(self));
+OUTPUT: RETVAL
+
+SV*
+to_perl(self)
+    cfish_Obj *self;
+CODE:
+    RETVAL = (SV*)CFISH_Obj_To_Host(self);
+OUTPUT: RETVAL
 END_XS_CODE
 
     my $binding = Clownfish::CFC::Binding::Perl::Class->new(
@@ -481,86 +820,84 @@ END_XS_CODE
         alias  => 'DESTROY',
         method => 'Destroy',
     );
-    $binding->exclude_method($_) for @hand_rolled;
     $binding->append_xs($xs_code);
     $binding->set_pod_spec($pod_spec);
 
     Clownfish::CFC::Binding::Perl::Class->register($binding);
 }
 
-sub bind_varray {
+sub bind_vector {
     my @hand_rolled = qw(
-        Shallow_Copy
-        Shift
-        Pop
-        Delete
         Store
-        Fetch
+    );
+
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $vector = Clownfish::Vector->new;
+    $vector->store($tick, $value);
+    my $value = $vector->fetch($tick);
+END_SYNOPSIS
+    my $constructor = <<'END_CONSTRUCTOR';
+    my $vector = Clownfish::Vector->new( capacity => 256 );
+END_CONSTRUCTOR
+    my $store_pod = <<'END_POD';
+=head2 store
+
+    $vector->store($tick, $elem)
+
+Store an element at index C<tick>, possibly displacing an existing element.
+END_POD
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor( sample => $constructor );
+    $pod_spec->add_method(
+        method => 'Store',
+        alias  => 'store',
+        pod    => $store_pod,
     );
 
     my $xs_code = <<'END_XS_CODE';
-MODULE = Clownfish   PACKAGE = Clownfish::VArray
+MODULE = Clownfish   PACKAGE = Clownfish::Vector
 
 SV*
-shallow_copy(self)
-    cfish_VArray *self;
+pop_raw(self)
+    cfish_Vector *self;
 CODE:
-    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_VA_Shallow_Copy(self));
+    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_Vec_Pop(self));
 OUTPUT: RETVAL
 
 SV*
-_clone(self)
-    cfish_VArray *self;
-CODE:
-    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_VA_Clone(self));
-OUTPUT: RETVAL
-
-SV*
-shift(self)
-    cfish_VArray *self;
-CODE:
-    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_VA_Shift(self));
-OUTPUT: RETVAL
-
-SV*
-pop(self)
-    cfish_VArray *self;
-CODE:
-    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_VA_Pop(self));
-OUTPUT: RETVAL
-
-SV*
-delete(self, tick)
-    cfish_VArray *self;
+delete_raw(self, tick)
+    cfish_Vector *self;
     uint32_t    tick;
 CODE:
-    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_VA_Delete(self, tick));
+    RETVAL = CFISH_OBJ_TO_SV_NOINC(CFISH_Vec_Delete(self, tick));
 OUTPUT: RETVAL
 
 void
 store(self, tick, value);
-    cfish_VArray *self;
+    cfish_Vector *self;
     uint32_t     tick;
     cfish_Obj    *value;
 PPCODE:
 {
     if (value) { CFISH_INCREF(value); }
-    CFISH_VA_Store_IMP(self, tick, value);
+    CFISH_Vec_Store_IMP(self, tick, value);
 }
 
 SV*
-fetch(self, tick)
-    cfish_VArray *self;
+fetch_raw(self, tick)
+    cfish_Vector *self;
     uint32_t     tick;
 CODE:
-    RETVAL = CFISH_OBJ_TO_SV(CFISH_VA_Fetch(self, tick));
+    RETVAL = CFISH_OBJ_TO_SV_INC(CFISH_Vec_Fetch(self, tick));
 OUTPUT: RETVAL
 END_XS_CODE
 
     my $binding = Clownfish::CFC::Binding::Perl::Class->new(
         parcel     => "Clownfish",
-        class_name => "Clownfish::VArray",
+        class_name => "Clownfish::Vector",
     );
+    $binding->set_pod_spec($pod_spec);
     $binding->exclude_method($_) for @hand_rolled;
     $binding->append_xs($xs_code);
 
@@ -568,29 +905,40 @@ END_XS_CODE
 }
 
 sub bind_class {
+    my $pod_spec = Clownfish::CFC::Binding::Perl::Pod->new;
+    my $synopsis = <<'END_SYNOPSIS';
+    my $class = Clownfish::Class->fetch_class('Foo::Bar');
+    my $subclass = Clownfish::Class->singleton('Foo::Bar::Jr', $class);
+END_SYNOPSIS
+    my $fetch_class_sample = <<'END_CONSTRUCTOR';
+    my $class = Clownfish::Class->fetch_class($class_name);
+END_CONSTRUCTOR
+    my $singleton_sample = <<'END_CONSTRUCTOR';
+    my $class = Clownfish::Class->singleton(
+        class_name => $class_name,
+        parent     => $parent,
+    );
+END_CONSTRUCTOR
+    $pod_spec->set_synopsis($synopsis);
+    $pod_spec->add_constructor(
+        alias  => 'fetch_class',
+        sample => $fetch_class_sample,
+    );
+    $pod_spec->add_constructor(
+        alias  => 'singleton',
+        sample => $singleton_sample,
+    );
+
     my $xs_code = <<'END_XS_CODE';
 MODULE = Clownfish   PACKAGE = Clownfish::Class
 
 SV*
-_get_registry()
-CODE:
-    if (cfish_Class_registry == NULL) {
-        cfish_Class_init_registry();
-    }
-    RETVAL = (SV*)CFISH_Obj_To_Host((cfish_Obj*)cfish_Class_registry);
-OUTPUT: RETVAL
-
-SV*
-fetch_class(unused_sv, class_name_sv)
+fetch_class(unused_sv, class_name)
     SV *unused_sv;
-    SV *class_name_sv;
+    cfish_String *class_name;
 CODE:
 {
-    STRLEN size;
-    char *ptr = SvPVutf8(class_name_sv, size);
-    cfish_StackString *class_name = CFISH_SSTR_WRAP_UTF8(ptr, size);
-    cfish_Class *klass
-        = cfish_Class_fetch_class((cfish_String*)class_name);
+    cfish_Class *klass = cfish_Class_fetch_class(class_name);
     CFISH_UNUSED_VAR(unused_sv);
     RETVAL = klass ? (SV*)CFISH_Class_To_Host(klass) : &PL_sv_undef;
 }
@@ -601,19 +949,22 @@ singleton(unused_sv, ...)
     SV *unused_sv;
 CODE:
 {
+    static const XSBind_ParamSpec param_specs[2] = {
+        XSBIND_PARAM("class_name", true),
+        XSBIND_PARAM("parent", false),
+    };
+    int32_t locations[2];
     cfish_String *class_name = NULL;
     cfish_Class  *parent     = NULL;
     cfish_Class  *singleton  = NULL;
-    bool args_ok
-        = XSBind_allot_params(&(ST(0)), 1, items,
-                              ALLOT_OBJ(&class_name, "class_name", 10, true,
-                                        CFISH_STRING, alloca(cfish_SStr_size())),
-                              ALLOT_OBJ(&parent, "parent", 6, false,
-                                        CFISH_CLASS, NULL),
-                              NULL);
     CFISH_UNUSED_VAR(unused_sv);
-    if (!args_ok) {
-        CFISH_RETHROW(CFISH_INCREF(cfish_Err_get_error()));
+    XSBind_locate_args(aTHX_ &(ST(0)), 1, items, param_specs, locations, 2);
+    class_name = (cfish_String*)XSBind_arg_to_cfish(
+            aTHX_ ST(locations[0]), "class_name", CFISH_STRING,
+            CFISH_ALLOCA_OBJ(CFISH_STRING));
+    if (locations[1] < items) {
+        parent = (cfish_Class*)XSBind_arg_to_cfish_nullable(
+                aTHX_ ST(locations[1]), "parent", CFISH_CLASS, NULL);
     }
     singleton = cfish_Class_singleton(class_name, parent);
     RETVAL = (SV*)CFISH_Class_To_Host(singleton);
@@ -625,6 +976,7 @@ END_XS_CODE
         parcel     => "Clownfish",
         class_name => "Clownfish::Class",
     );
+    $binding->set_pod_spec($pod_spec);
     $binding->append_xs($xs_code);
 
     Clownfish::CFC::Binding::Perl::Class->register($binding);
