@@ -20,16 +20,13 @@ package Clownfish;
 
 use 5.008003;
 
-our $VERSION = '0.004004';
+our $VERSION = '0.004099_001';
 $VERSION = eval $VERSION;
+our $MAJOR_VERSION = 0.004000;
 
 use Exporter 'import';
 BEGIN {
-    our @EXPORT_OK = qw(
-        to_clownfish
-        to_perl
-        kdump
-        );
+    our @EXPORT_OK = qw( to_clownfish );
 }
 
 # On most UNIX variants, this flag makes DynaLoader pass RTLD_GLOBAL to
@@ -41,22 +38,14 @@ BEGIN {
     require DynaLoader;
     our @ISA = qw( DynaLoader );
     # This loads a large number of disparate subs.
-    bootstrap Clownfish '0.4.4';
-}
-
-sub kdump {
-    require Data::Dumper;
-    my $kdumper = Data::Dumper->new( [@_] );
-    $kdumper->Sortkeys( sub { return [ sort keys %{ $_[0] } ] } );
-    $kdumper->Indent(1);
-    warn $kdumper->Dump;
+    bootstrap Clownfish '0.4.99_1';
 }
 
 sub error {$Clownfish::Err::error}
 
 {
     package Clownfish::Util::StringHelper;
-    our $VERSION = '0.004004';
+    our $VERSION = '0.004099_001';
     $VERSION = eval $VERSION;
     BEGIN {
         push our @ISA, 'Exporter';
@@ -72,19 +61,10 @@ sub error {$Clownfish::Err::error}
     }
 }
 
-{
-    package Clownfish::LockFreeRegistry;
-    our $VERSION = '0.004004';
-    $VERSION = eval $VERSION;
-    no warnings 'redefine';
-    sub DESTROY { }    # leak all
-}
-
 {
     package Clownfish::Obj;
-    our $VERSION = '0.004004';
+    our $VERSION = '0.004099_001';
     $VERSION = eval $VERSION;
-    use Clownfish qw( to_clownfish to_perl );
     use Carp qw( confess );
     # Clownfish objects are not thread-safe.
     sub CLONE_SKIP { 1; }
@@ -100,7 +80,7 @@ sub error {$Clownfish::Err::error}
 
 {
     package Clownfish::Class;
-    our $VERSION = '0.004004';
+    our $VERSION = '0.004099_001';
     $VERSION = eval $VERSION;
 
     sub _find_parent_class {
@@ -117,7 +97,7 @@ sub error {$Clownfish::Err::error}
         no strict 'refs';
         my $stash = \%{"$package\::"};
         my $methods
-            = Clownfish::VArray->new( capacity => scalar keys %$stash );
+            = Clownfish::Vector->new( capacity => scalar keys %$stash );
         while ( my ( $symbol, $glob ) = each %$stash ) {
             next if ref $glob;
             next unless *$glob{CODE};
@@ -137,53 +117,22 @@ sub error {$Clownfish::Err::error}
     }
 
     no warnings 'redefine';
+    sub CLONE_SKIP { 0; }
     sub DESTROY { }    # leak all
 }
 
 {
     package Clownfish::Method;
-    our $VERSION = '0.004004';
+    our $VERSION = '0.004099_001';
     $VERSION = eval $VERSION;
     no warnings 'redefine';
+    sub CLONE_SKIP { 0; }
     sub DESTROY { }    # leak all
 }
 
-{
-    package Clownfish::ViewByteBuf;
-    our $VERSION = '0.004004';
-    $VERSION = eval $VERSION;
-    use Carp;
-    sub new { confess "ViewByteBuf objects can only be created from C." }
-}
-
-{
-    package Clownfish::String;
-    our $VERSION = '0.004004';
-    $VERSION = eval $VERSION;
-
-    {
-        # Defeat obscure bugs in the XS auto-generation by redefining clone().
-        # (Because of how the typemap works for String*,
-        # the auto-generated methods return UTF-8 Perl scalars rather than
-        # actual String objects.)
-        no warnings 'redefine';
-        sub clone { shift->_clone(@_) }
-    }
-}
-
-{
-    package Clownfish::StackString;
-    our $VERSION = '0.004004';
-    $VERSION = eval $VERSION;
-    use Carp;
-    sub new { confess "StackString objects can only be created from C." }
-    no warnings 'redefine';
-    sub DESTROY { }
-}
-
 {
     package Clownfish::Err;
-    our $VERSION = '0.004004';
+    our $VERSION = '0.004099_001';
     $VERSION = eval $VERSION;
     sub do_to_string { shift->to_string }
     use Scalar::Util qw( blessed );
@@ -221,11 +170,13 @@ sub error {$Clownfish::Err::error}
 }
 
 {
-    package Clownfish::VArray;
-    our $VERSION = '0.004004';
+    package Clownfish::Boolean;
+    our $VERSION = '0.004099_001';
     $VERSION = eval $VERSION;
-    no warnings 'redefine';
-    sub clone       { CORE::shift->_clone }
+    use Exporter 'import';
+    our @EXPORT_OK = qw( $true_singleton $false_singleton );
+    our $true_singleton  = Clownfish::Boolean->singleton(1);
+    our $false_singleton = Clownfish::Boolean->singleton(0);
 }
 
 1;
@@ -238,7 +189,7 @@ Clownfish - Apache Clownfish symbiotic object system.
 
 =head1 VERSION
 
-0.4.4
+0.4.99_1
 
 =head1 DESCRIPTION
 
