@@ -13,9 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-package Clownfish::String;
+package Clownfish::CharBuf;
 use Clownfish;
-our $VERSION = '0.004004';
+our $VERSION = '0.004099_001';
 $VERSION = eval $VERSION;
 
 1;
